﻿namespace TestLibrary
{
    /// <summary>
    /// 폰트 스타일
    /// </summary>
    public class FontStyle
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 추가할 스타일
        /// </summary>
        private uint styleToAdd;

        /// <summary>
        /// 제거할 스타일
        /// </summary>
        private uint styleToRemove;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 비어있는지 여부 - IsEmpty

        /// <summary>
        /// 비어있는지 여부
        /// </summary>
        public bool IsEmpty
        {
            get
            {
                return this.styleToAdd == 0 && this.styleToRemove == 0;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Internal

        #region 생성자 -  FontStyle()

        /// <summary>
        /// 생성자
        /// </summary>
        internal FontStyle()
        {
            this.styleToAdd    = 0;
            this.styleToRemove = 0;
        }

        #endregion
        #region 생성자 - FontStyle(source)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="source">소스 폰트 스타일</param>
        internal FontStyle(FontStyle source)
        {
            this.styleToAdd    = source.styleToAdd;
            this.styleToRemove = source.styleToRemove;
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 스타일 추가하기 - AddStyle(fontStyleFlag)

        /// <summary>
        /// 스타일 추가하기
        /// </summary>
        /// <param name="fontStyleFlag">폰트 스타일 플래그</param>
        public void AddStyle(FontStyleFlag fontStyleFlag)
        {
            this.styleToAdd    |=   (uint)fontStyleFlag;
            this.styleToRemove &= ~((uint)fontStyleFlag);
        }

        #endregion
        #region 스타일 제거하기 - RemoveStyle(fontStyleFlag)

        /// <summary>
        /// 스타일 제거하기
        /// </summary>
        /// <param name="fontStyleFlag">폰트 스타일 플래그</param>
        public void RemoveStyle(FontStyleFlag fontStyleFlag)
        {
            this.styleToAdd    &= ~((uint)fontStyleFlag);
            this.styleToRemove |=   (uint)fontStyleFlag;
        }

        #endregion

        #region 추가할 스타일 포함 여부 구하기 - ContainsStyleToAdd(fontStyleFlag)

        /// <summary>
        /// 추가할 스타일 포함 여부 구하기
        /// </summary>
        /// <param name="fontStyleFlag">폰트 스타일 플래그</param>
        /// <returns>추가할 스타일 포함 여부</returns>
        public bool ContainsStyleToAdd(FontStyleFlag fontStyleFlag)
        {
            if((this.styleToAdd & (uint)fontStyleFlag) > 0)
            {
                return true;
            }

            return false;
        }

        #endregion
        #region 제거할 스타일 포함 여부 구하기 - ContainsStyleToRemove(fontStyleFlag)

        /// <summary>
        /// 제거할 스타일 포함 여부 구하기
        /// </summary>
        /// <param name="fontStyleFlag">폰트 스타일 플래그</param>
        /// <returns>제거할 스타일 포함 여부</returns>
        public bool ContainsStyleToRemove(FontStyleFlag fontStyleFlag)
        {
            if((this.styleToRemove & (uint)fontStyleFlag) > 0)
            {
                return true;
            }

            return false;
        }

        #endregion
    }
}