﻿using System;
using System.Drawing;

namespace TestLibrary
{
    /// <summary>
    /// RTF 색상
    /// </summary>
    public class RTFColor
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 색상
        /// </summary>
        private int color;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Internal

        #region 적색 - Red

        /// <summary>
        /// 적색
        /// </summary>
        internal string Red
        {
            get
            {
                return ((this.color >> 16) % 256).ToString();
            }
        }

        #endregion
        #region 녹색 - Green

        /// <summary>
        /// 녹색
        /// </summary>
        internal string Green
        {
            get
            {
                return ((this.color >> 8) % 256).ToString();
            }
        }

        #endregion
        #region 청색 - Blue

        /// <summary>
        /// 청색
        /// </summary>
        internal string Blue
        {
            get
            {
                return (this.color % 256).ToString();
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - RTFColor()

        /// <summary>
        /// 생성자
        /// </summary>
        public RTFColor()
        {
            this.color = 0;
        }

        #endregion
        #region 생성자 - RTFColor(hex)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="hex">16진수 색상값</param>
        public RTFColor(string hex)
        {
            if(hex == null || hex.Length != 6)
            {
                throw new Exception("String parameter hex should be of length 6.");
            }

            hex = hex.ToUpper();

            for(int i = 0; i < hex.Length; i++)
            {
                if(!char.IsDigit(hex[i]) && (hex[i] < 'A' || hex[i] > 'F'))
                {
                    throw new Exception("Characters of parameter hex should be in [0-9,A-F,a-f]");
                }
            }

            byte red   = Convert.ToByte(hex.Substring(0, 2), 16);
            byte green = Convert.ToByte(hex.Substring(2, 2), 16);
            byte blue  = Convert.ToByte(hex.Substring(4, 2), 16);

            this.color = (red << 16) + (green << 8) + blue;
        }

        #endregion
        #region 생성자 - RTFColor(color)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="color">색상</param>
        public RTFColor(Color color)
        {
            this.color = ( color.R << 16 ) + ( color.G << 8 ) + color.B;
        }

        #endregion
        #region 생성자 - RTFColor(red, green, blue)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="red">적색</param>
        /// <param name="green">녹색</param>
        /// <param name="blue">청색</param>
        public RTFColor(byte red, byte green, byte blue)
        {
            this.color = (red << 16) + (green << 8) + blue;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 해시 코드 구하기 - GetHashCode()

        /// <summary>
        /// 해시 코드 구하기
        /// </summary>
        /// <returns>해시 코드</returns>
        public override int GetHashCode()
        {
            return this.color;
        }

        #endregion
        #region 동일 여부 구하기 - Equals(source)

        /// <summary>
        /// 동일 여부 구하기
        /// </summary>
        /// <param name="source">소스 객체</param>
        /// <returns>동일 여부</returns>
        public override bool Equals(object source)
        {
            RTFColor rtfColor = (RTFColor)source;

            return (rtfColor.color == this.color);
        }

        #endregion
    }
}