using System.Collections.Generic;
using System.Drawing;
using System.Text;
using System.IO;

namespace TestLibrary
{
    /// <summary>
    /// RTF 
    /// </summary>
    public class RTFDocument : RTFBlockList
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        ///  ũ
        /// </summary>
        private PaperSize paperSize;

        /// <summary>
        ///  
        /// </summary>
        private PaperOrientation paperOrientation;

        /// <summary>
        /// 
        /// </summary>
        private Margins margins;

        /// <summary>
        ///  
        /// </summary>
        private LCID lcid;

        /// <summary>
        /// Ʈ Ʈ
        /// </summary>
        private List<string> fontList;

        /// <summary>
        ///  Ʈ
        /// </summary>
        private List<RTFColor> colorList;

        /// <summary>
        /// 
        /// </summary>
        private RTFHeaderFooter header;

        /// <summary>
        /// ٴڱ
        /// </summary>
        private RTFHeaderFooter footer;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region  - Margins

        /// <summary>
        /// 
        /// </summary>
        public Margins Margins
        {
            get
            {
                return this.margins;
            }
            set
            {
                this.margins = value;
            }
        }

        #endregion
        #region  - Header

        /// <summary>
        /// 
        /// </summary>
        public RTFHeaderFooter Header
        {
            get
            {
                if(this.header == null)
                {
                    this.header = new RTFHeaderFooter(HeaderFooterType.Header);
                }

                return this.header;
            }
        }

        #endregion
        #region ٴڱ - Footer

        /// <summary>
        /// ٴڱ
        /// </summary>
        public RTFHeaderFooter Footer
        {
            get
            {
                if(this.footer == null)
                {
                    this.footer = new RTFHeaderFooter(HeaderFooterType.Footer);
                }

                return this.footer;
            }
        }

        #endregion
        #region Ʈ Ʈ  - DefaultFontDescriptor

        /// <summary>
        /// Ʈ Ʈ 
        /// </summary>
        public FontDescriptor DefaultFontDescriptor
        {
            get
            {
                return new FontDescriptor(0);
            }
        }

        #endregion
        #region Ʈ   - DefaultColorDescriptor

        /// <summary>
        /// Ʈ  
        /// </summary>
        public ColorDescriptor DefaultColorDescriptor
        {
            get
            {
                return  new ColorDescriptor(0);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region  - RTFDocument(paperSize, paperOrientation, lcid)

        /// <summary>
        /// 
        /// </summary>
        /// <param name="paperSize"> ũ</param>
        /// <param name="paperOrientation"> </param>
        /// <param name="lcid"> </param>
        public RTFDocument(PaperSize paperSize, PaperOrientation paperOrientation, LCID lcid)
        {
            this.paperSize        = paperSize;
            this.paperOrientation = paperOrientation;
            this.margins          = new Margins();

            if(this.paperOrientation == PaperOrientation.Portrait)
            {
                this.margins[Direction.Top   ] = DefaultValue.SmallMargin;
                this.margins[Direction.Right ] = DefaultValue.LargeMargin;
                this.margins[Direction.Bottom] = DefaultValue.SmallMargin;
                this.margins[Direction.Left  ] = DefaultValue.LargeMargin;
            }
            else
            {
                this.margins[Direction.Top   ] = DefaultValue.LargeMargin;
                this.margins[Direction.Right ] = DefaultValue.SmallMargin;
                this.margins[Direction.Bottom] = DefaultValue.LargeMargin;
                this.margins[Direction.Left  ] = DefaultValue.SmallMargin;
            }

            this.lcid = lcid;

            this.fontList = new List<string>();

            this.fontList.Add(DefaultValue.Font);

            this.colorList = new List<RTFColor>();

            this.colorList.Add(new RTFColor());

            this.header = null;
            this.footer = null;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region Ʈ Ʈ ϱ - SetDefaultFont(fontName)

        /// <summary>
        /// Ʈ Ʈ ϱ
        /// </summary>
        /// <param name="fontName">Ʈ</param>
        public void SetDefaultFont(string fontName)
        {
            this.fontList[0] = fontName;
        }

        #endregion
        #region Ʈ  ϱ - CreateFontDescriptor(fontName)

        /// <summary>
        /// Ʈ  ϱ
        /// </summary>
        /// <param name="fontName">Ʈ</param>
        /// <returns>Ʈ </returns>
        public FontDescriptor CreateFontDescriptor(string fontName)
        {
            if(this.fontList.Contains(fontName))
            {
                return new FontDescriptor(this.fontList.IndexOf(fontName));
            }

            this.fontList.Add(fontName);

            return new FontDescriptor(this.fontList.IndexOf(fontName));
        }

        #endregion
        #region   ϱ - CreateColorDescriptor(rtfColor)

        /// <summary>
        ///   ϱ
        /// </summary>
        /// <param name="rtfColor">RTF </param>
        /// <returns> </returns>
        public ColorDescriptor CreateColorDescriptor(RTFColor rtfColor)
        {
            if(this.colorList.Contains(rtfColor))
            {
                return new ColorDescriptor(this.colorList.IndexOf(rtfColor));
            }

            this.colorList.Add(rtfColor);

            return new ColorDescriptor(this.colorList.IndexOf(rtfColor));
        }

        #endregion
        #region   ϱ - CreateColorDescriptor(color)

        /// <summary>
        ///   ϱ
        /// </summary>
        /// <param name="color"></param>
        /// <returns> </returns>
        public ColorDescriptor CreateColorDescriptor(Color color)
        {
            RTFColor rtfColor = new RTFColor(color.R, color.G, color.B);

            return CreateColorDescriptor(rtfColor);
        }

        #endregion
        #region ̺ ߰ϱ - AddTable(rowCount, columnCount, fontSize)

        /// <summary>
        /// ̺ ߰ϱ
        /// </summary>
        /// <param name="rowCount"> </param>
        /// <param name="columnCount"> </param>
        /// <param name="fontSize">Ʈ ũ</param>
        /// <returns>RTF ̺</returns>
        public RTFTable AddTable(int rowCount, int columnCount, float fontSize)
        {
            float width;

            if(this.paperOrientation == PaperOrientation.Portrait)
            {
                width = RTFUtility.GetPaperWidthInPoint(this.paperSize, this.paperOrientation) -
                    this.margins[Direction.Left] - this.margins[Direction.Right];
            }
            else
            {
                width = RTFUtility.GetPaperHeightInPoint(this.paperSize, this.paperOrientation) -
                    this.margins[Direction.Left] - this.margins[Direction.Right];
            }

            return base.AddTable(rowCount, columnCount, width, fontSize);
        }

        #endregion
        #region ϱ - Render()

        /// <summary>
        /// ϱ
        /// </summary>
        /// <returns>RTF ڿ</returns>
        public override string Render()
        {
            StringBuilder stringBuilder = new StringBuilder();

            stringBuilder.AppendLine(@"{\rtf1\ansi\deff0");
            stringBuilder.AppendLine();

            stringBuilder.AppendLine(@"{\fonttbl");

            for(int i = 0; i < this.fontList.Count; i++)
            {
                stringBuilder.AppendLine(@"{\f" + i + " " + RTFUtility.ConvertToUnicodeEncoding(this.fontList[i].ToString()) + ";}");
            }

            stringBuilder.AppendLine("}");
            stringBuilder.AppendLine();

            stringBuilder.AppendLine(@"{\colortbl");
            stringBuilder.AppendLine(";");

            for(int i = 1; i < this.colorList.Count; i++)
            {
                RTFColor color = this.colorList[i];

                stringBuilder.AppendLine(@"\red" + color.Red + @"\green" + color.Green + @"\blue" + color.Blue + ";");
            }

            stringBuilder.AppendLine("}");
            stringBuilder.AppendLine();

            stringBuilder.AppendLine
            (
                @"\deflang"                                   +
                (int)this.lcid                                +
                @"\plain\fs"                                  +
                RTFUtility.DoubleValue(DefaultValue.FontSize) +
                @"\widowctrl\hyphauto\ftnbj"
            );

            stringBuilder.AppendLine
            (
                @"\paperw"                                                            +
                RTFUtility.GetPaperWidthInTwip(this.paperSize, this.paperOrientation) +
                @"\paperh"                                                            +
                RTFUtility.GetPaperHeightInTwip(this.paperSize, this.paperOrientation)
            );

            stringBuilder.AppendLine(@"\margt" + RTFUtility.ConvertPointToTwip(this.margins[Direction.Top   ]));
            stringBuilder.AppendLine(@"\margr" + RTFUtility.ConvertPointToTwip(this.margins[Direction.Right ]));
            stringBuilder.AppendLine(@"\margb" + RTFUtility.ConvertPointToTwip(this.margins[Direction.Bottom]));
            stringBuilder.AppendLine(@"\margl" + RTFUtility.ConvertPointToTwip(this.margins[Direction.Left  ]));

            if(this.paperOrientation == PaperOrientation.Landscape)
            {
                stringBuilder.AppendLine(@"\landscape");
            }

            if(this.header != null)
            {
                stringBuilder.Append(this.header.Render());
            }

            if(this.footer != null)
            {
                stringBuilder.Append(this.footer.Render());
            }

            stringBuilder.AppendLine();

            stringBuilder.Append(base.Render());

            stringBuilder.AppendLine("}");

            return stringBuilder.ToString();
        }

        #endregion
        #region ϱ - Save(filePath)

        /// <summary>
        /// ϱ
        /// </summary>
        /// <param name="filePath"> </param>
        public void Save(string filePath)
        {
            StreamWriter writer = new StreamWriter(filePath);

            writer.Write(Render());

            writer.Close();
        }

        #endregion
    }
}