using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Text;
using System.IO;

namespace TestLibrary
{
    /// <summary>
    /// RTF ̹
    /// </summary>
    public class RTFImage : RTFBlock
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// ̹  Ÿ
        /// </summary>
        private ImageFileType imageFieldType;

        /// <summary>
        /// ̹ Ʈ 迭
        /// </summary>
        private byte[] imageByteArray;

        /// <summary>
        ///  
        /// </summary>
        private HorizontalAlignment horizontalAlignment;

        /// <summary>
        /// 
        /// </summary>
        private Margins margins;

        /// <summary>
        /// ʺ
        /// </summary>
        private float width;

        /// <summary>
        /// 
        /// </summary>
        private float height;

        /// <summary>
        /// Ⱦ  
        /// </summary>
        private bool keepAspectRatio;

        /// <summary>
        ///  
        /// </summary>
        private string blockHead;

        /// <summary>
        ///  
        /// </summary>
        private string blockTail;

        /// <summary>
        ///    
        /// </summary>
        private bool startNewPage;

        /// <summary>
        ///    
        /// </summary>
        private bool startNewParagraph;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region   - HorizontalAlignment

        /// <summary>
        ///  
        /// </summary>
        public override HorizontalAlignment HorizontalAlignment
        {
            get
            {
                return this.horizontalAlignment;
            }
            set
            {
                this.horizontalAlignment = value;
            }
        }

        #endregion
        #region  - Margins

        /// <summary>
        /// 
        /// </summary>
        public override Margins Margins
        {
            get
            {
                return this.margins;
            }
        }

        #endregion
        #region     - StartNewPage

        /// <summary>
        ///    
        /// </summary>
        public override bool StartNewPage
        {
            get
            {
                return this.startNewPage;
            }
            set
            {
                this.startNewPage = value;
            }
        }

        #endregion
        #region     - StartNewParagraph

        /// <summary>
        ///    
        /// </summary>
        public bool StartNewParagraph
        {
            get
            {
                return this.startNewParagraph;
            }
            set
            {
                this.startNewParagraph = value;
            }
        }

        #endregion
        #region ʺ - Width

        /// <summary>
        /// ʺ
        /// </summary>
        public float Width
        {
            get
            {
                return this.width;
            }
            set
            {
                if(this.keepAspectRatio && this.width > 0)
                {
                    float ratio = this.height / this.width;

                    this.height = value * ratio;
                }

                this.width = value;
            }
        }

        #endregion
        #region  - Height

        /// <summary>
        /// 
        /// </summary>
        public float Height
        {
            get
            {
                return this.height;
            }
            set
            {
                if(this.keepAspectRatio && this.height > 0)
                {
                    float ratio = this.width / this.height;

                    this.width = value * ratio;
                }

                this.height = value;
            }
        }

        #endregion
        #region Ⱦ   - KeepAspectRatio

        /// <summary>
        /// Ⱦ  
        /// </summary>
        public bool KeepAspectRatio
        {
            get
            {
                return this.keepAspectRatio;
            }
            set
            {
                this.keepAspectRatio = value;
            }
        }

        #endregion
        #region Ʈ   - DefaultCharFormat

        /// <summary>
        /// Ʈ  
        /// </summary>
        public override RTFCharFormat DefaultCharFormat
        {
            get
            {
                return null;
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Internal

        #region   - BlockHead

        /// <summary>
        ///  
        /// </summary>
        internal override string BlockHead
        {
            set
            {
                this.blockHead = value;
            }
        }

        #endregion
        #region   - BlockTail

        /// <summary>
        ///  
        /// </summary>
        internal override string BlockTail
        {
            set
            {
                this.blockTail = value;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Internal

        #region  - RTFImage(memoryStream)

        /// <summary>
        /// 
        /// </summary>
        /// <param name="memoryStream">޸ Ʈ</param>
        internal RTFImage(MemoryStream memoryStream)
        {
            this.horizontalAlignment = HorizontalAlignment.Left;
            this.margins             = new Margins();
            this.keepAspectRatio     = true;
            this.blockHead           = @"{\pard";
            this.blockTail           = @"}";
            this.startNewPage        = false;
            this.startNewParagraph   = false;

            this.imageByteArray = memoryStream.ToArray();

            Image image = Image.FromStream(memoryStream);

            this.width  = (image.Width  / image.HorizontalResolution) * 72;
            this.height = (image.Height / image.VerticalResolution  ) * 72;

            if(image.RawFormat.Equals(ImageFormat.Png))
            {
                this.imageFieldType = ImageFileType.PNG;
            }
            else if(image.RawFormat.Equals(ImageFormat.Jpeg))
            {
                this.imageFieldType = ImageFileType.JPEG;
            }
            else if(image.RawFormat.Equals(ImageFormat.Gif))
            {
                this.imageFieldType = ImageFileType.GIF;
            }
            else
            {
                throw new Exception($"Image format is not supported : {image.RawFormat}");
            }
        }

        #endregion
        #region  - RTFImage(filePath, type)

        /// <summary>
        /// 
        /// </summary>
        /// <param name="filePath"> </param>
        /// <param name="type">̹  Ÿ</param>
        internal RTFImage(string filePath, ImageFileType type)
        {
            this.imageFieldType      = type;
            this.horizontalAlignment = HorizontalAlignment.None;
            this.margins             = new Margins();
            this.keepAspectRatio     = true;
            this.blockHead           = @"{\pard";
            this.blockTail           = @"}";
            this.startNewPage        = false;
            this.startNewParagraph   = false;
            
            Image image = Image.FromFile(filePath);

            this.width  = (image.Width  / image.HorizontalResolution) * 72;
            this.height = (image.Height / image.VerticalResolution  ) * 72;

            using(MemoryStream memoryStream = new MemoryStream())
            {
                image.Save(memoryStream, image.RawFormat);

                this.imageByteArray = memoryStream.ToArray();
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region ϱ - Render()

        /// <summary>
        /// ϱ
        /// </summary>
        /// <returns>RTF ڿ</returns>
        public override string Render()
        {
            StringBuilder stringBuilder = new StringBuilder(this.blockHead);

            if(this.startNewPage)
            {
                stringBuilder.Append(@"\pagebb");
            }

            if(this.margins[Direction.Top] >= 0)
            {
                stringBuilder.Append(@"\sb" + RTFUtility.ConvertPointToTwip(this.margins[Direction.Top]));
            }

            if(this.margins[Direction.Bottom] >= 0)
            {
                stringBuilder.Append(@"\sa" + RTFUtility.ConvertPointToTwip(this.margins[Direction.Bottom]));
            }

            if(this.margins[Direction.Left] >= 0)
            {
                stringBuilder.Append(@"\li" + RTFUtility.ConvertPointToTwip(this.margins[Direction.Left]));
            }

            if(this.margins[Direction.Right] >= 0)
            {
                stringBuilder.Append(@"\ri" + RTFUtility.ConvertPointToTwip(this.margins[Direction.Right]));
            }

            switch(this.horizontalAlignment)
            {
                case HorizontalAlignment.Left   : stringBuilder.Append(@"\ql"); break;
                case HorizontalAlignment.Right  : stringBuilder.Append(@"\qr"); break;
                case HorizontalAlignment.Center : stringBuilder.Append(@"\qc"); break;
            }

            stringBuilder.AppendLine();

            stringBuilder.Append(@"{\*\shppict{\pict");

            if(this.imageFieldType == ImageFileType.JPEG)
            {
                stringBuilder.Append(@"\jpegblip");
            }
            else if(this.imageFieldType == ImageFileType.PNG || this.imageFieldType == ImageFileType.GIF)
            {
                stringBuilder.Append(@"\pngblip");
            }
            else
            {
                throw new Exception("Image type not supported.");
            }

            if(this.height > 0)
            {
                stringBuilder.Append(@"\pichgoal" + RTFUtility.ConvertPointToTwip(this.height));
            }

            if(this.width > 0)
            {
                stringBuilder.Append(@"\picwgoal" + RTFUtility.ConvertPointToTwip(this.width));
            }

            stringBuilder.AppendLine();

            stringBuilder.AppendLine(ExtractImage());

            stringBuilder.AppendLine("}}");

            if(this.startNewParagraph)
            {
                stringBuilder.Append(@"\par");
            }

            stringBuilder.AppendLine(this.blockTail);

            return stringBuilder.ToString();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region ̹ ϱ - ExtractImage()

        /// <summary>
        /// ̹ ϱ
        /// </summary>
        /// <returns>̹ RTF ڿ</returns>
        private string ExtractImage()
        {
            StringBuilder stringBuilder = new StringBuilder();

            for(int i = 0; i < this.imageByteArray.Length; i++)
            {
                if(i != 0 && i % 60 == 0)
                {
                    stringBuilder.AppendLine();
                }

                stringBuilder.AppendFormat("{0:x2}", this.imageByteArray[i]);
            }

            return stringBuilder.ToString();
        }

        #endregion
    }
}